<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Custom Form Newsletter Subscription
 */

namespace Amasty\CustomFormNewsletterSubscription\Observer\Frontend\Form;

use Amasty\Base\Model\Di\Wrapper as DiWrapper;
use Amasty\Customform\Api\Data\AnswerInterface;
use Amasty\Customform\Api\Data\FormInterface;
use Amasty\Customform\Model\Utils\CustomerInfo;
use Amasty\CustomFormNewsletterSubscription\Model\ConfigProvider;
use Amasty\CustomFormNewsletterSubscription\Model\Form\Rendering\FormJson\AddSubscriptionField;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Message\Manager as MessageManager;
use Magento\Framework\Module\Manager as ModuleManager;
use Magento\Newsletter\Model\Subscriber;
use Magento\Newsletter\Model\SubscriberFactory;
use Magento\Newsletter\Model\SubscriptionManagerInterface;
use Magento\Store\Model\StoreManagerInterface;

class SubscribeForNews implements ObserverInterface
{
    /**
     * @var RequestInterface
     */
    private $request;

    /**
     * @var ModuleManager
     */
    private $moduleManager;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var ConfigProvider
     */
    private $configProvider;

    /**
     * @var CustomerInfo
     */
    private $customerInfo;

    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;

    /**
     * @var DiWrapper|SubscriptionManagerInterface
     */
    private $subscriptionManager;

    /**
     * @var MessageManager
     */
    private $messageManager;

    /**
     * @var DiWrapper|SubscriberFactory
     */
    private $subscriptionFactory;

    public function __construct(
        RequestInterface $request,
        ModuleManager $moduleManager,
        StoreManagerInterface $storeManager,
        ConfigProvider $configProvider,
        CustomerInfo $customerInfo,
        CustomerRepositoryInterface $customerRepository,
        DiWrapper $subscriptionManager,
        MessageManager $messageManager,
        DiWrapper $subscriptionFactory
    ) {
        $this->request = $request;
        $this->moduleManager = $moduleManager;
        $this->storeManager = $storeManager;
        $this->configProvider = $configProvider;
        $this->customerInfo = $customerInfo;
        $this->customerRepository = $customerRepository;
        $this->subscriptionManager = $subscriptionManager;
        $this->messageManager = $messageManager;
        $this->subscriptionFactory = $subscriptionFactory;
    }

    public function execute(Observer $observer): void
    {
        /** @var AnswerInterface $answer */
        $answer = $observer->getEvent()->getAnswer();
        /** @var FormInterface $form */
        $form = $observer->getEvent()->getForm();

        if ($form
            && $answer
            && $this->moduleManager->isEnabled('Magento_Newsletter')
            && $form->isSubscriptionEnabled()
            && ($form->isSubscriptionRequired()
                || $this->request->getParam(AddSubscriptionField::SUBSCRIPTION_FIELD))
            && ($this->configProvider->isNewsletterSubscribeAllowedForGuest() || $this->customerInfo->isLoggedIn())
        ) {
            if (!$this->validateEmailAvailable($answer->getRecipientEmail())) {
                return;
            }

            if ($this->isAlreadySubscribed($answer->getRecipientEmail())) {
                $this->messageManager->addErrorMessage(__('This email address is already subscribed.'));
                return;
            }

            if ($answer->getCustomerId()
                && $this->customerInfo->getCurrentCustomer()->getEmail() === $answer->getRecipientEmail()
            ) {
                $this->subscriptionManager->subscribeCustomer($answer->getCustomerId(), (int)$answer->getStoreId());
            } else {
                $this->subscriptionManager->subscribe($answer->getRecipientEmail(), (int)$answer->getStoreId());
            }
        }
    }

    private function validateEmailAvailable(string $email): bool
    {
        $websiteId = (int)$this->storeManager->getStore()->getWebsiteId();
        if ($this->customerInfo->isLoggedIn()
            && ($this->customerInfo->getCurrentCustomer()->getEmail() !== $email
                && !$this->isEmailAvailable($email, $websiteId))
        ) {
            $this->messageManager->addErrorMessage(__('This email address is already assigned to another user.'));
            return false;
        }

        return true;
    }

    private function isEmailAvailable(string $customerEmail, int $websiteId): bool
    {
        try {
            $this->customerRepository->get($customerEmail, $websiteId);
            return false;
        } catch (NoSuchEntityException $e) {
            return true;
        }
    }

    private function isAlreadySubscribed(string $email): bool
    {
        $websiteId = (int)$this->storeManager->getStore()->getWebsiteId();
        /** @var Subscriber $subscriber */
        $subscriber = $this->subscriptionFactory->create();
        $subscriber->loadBySubscriberEmail($email, $websiteId);
        return $subscriber->getId()
            && (int)$subscriber->getSubscriberStatus() === Subscriber::STATUS_SUBSCRIBED;
    }
}
